class Breakpoint:

    __slots__ = "is_enabled"
    """
    Base class for beakpoints, intended to be subclassed.

    Attributes
    ----------
    is_enabled : bool
        Used to check and toggle if a breakpoint is on or off.
    """

    def __init__(self):
        self.is_enabled = True

    def is_break(self) -> bool:
        """Check if the breakpoint has been reached.

        Returns
        -------
        bool
            ``True`` if the breakpoint has been reached, ``False`` otherwise.
        """
        return False

    def set_enabled(self, is_enabled: bool) -> None:
        """Toggle the breakpoint on or off.

        Parameters
        ----------
        is_enabled : bool
            ``True`` to toggle on, ``False`` to toggle off.
        """
        self.is_enabled = is_enabled
