from collections.abc import Callable

from simudator.core.breakpoint import Breakpoint


class LambdaBreakpoint(Breakpoint):

    __slots__ = ("_lambda_func", "_kwargs")
    """
    A breakpoint that uses a given function to decide whether the breakpoint
    has been reached or not.

    Parameters
    ----------
    lambda_func : Callable
        The function used to decide if the breakpoint has been reached.

        ``lambda_func(**kwargs) -> bool``
        Should return ``True`` if the breakpoint has been reached, ``False``
        otherwise.
    **kwargs
        Arguments to pass to `lambda_func`.
    """

    def __init__(self, lambda_func: Callable[..., bool], **kwargs) -> None:
        super().__init__()
        self._lambda_func = lambda_func
        self._kwargs = kwargs

    def is_break(self):
        return self._lambda_func(**self._kwargs)

    def __str__(self) -> str:
        s = f"Lambda {self._lambda_func.__name__} with kwargs {self._kwargs}"
        return s
