from typing import Any

from simudator.core.breakpoint import Breakpoint
from simudator.core.modules import Memory


class MemoryBreakpoint(Breakpoint):

    __slots__ = ("_memory", "_address", "_value")

    """
    A breakpoint class for memory modules to break when a certain address
    of the memory has a certain value.

    Parameters
    ----------
    memory : Memory
        Memory module to check the contents of.
    address : int
        Address of the memory module to break at.
    value : Any
        Value of the address that triggers the breakpoint.
    """

    def __init__(self, memory: Memory, address: int, value: Any) -> None:
        super().__init__()
        self._memory = memory
        self._address = address
        self._value = value

    def is_break(self):
        memory_content = self._memory.get_state()["memory"]
        return memory_content[self._address] == self._value

    def __str__(self) -> str:
        return f"{self._memory.name} {self._address}: {self._value}"
