from typing import Any

from simudator.core.breakpoint import Breakpoint
from simudator.core.module import Module


class StateBreakpoint(Breakpoint):

    __slots__ = ("_module", "_state", "_value")
    """
    A breakpoint class to break when the state variable for a module reaches
    a specified value.

    Parameters
    ----------
    module : Module
        Module to check the state of.
    state : str
        Name of the state variable of the module to check the value of.
    value : Any
        Value of the state variable to break at.
    """

    def __init__(self, module: Module, state: str, value: Any) -> None:
        super().__init__()
        self._module = module
        self._state = state
        self._value = value

    def is_break(self):
        return self._module.get_state()[self._state] == self._value

    def __str__(self) -> str:
        return self._module.name + " " + self._state + " == " + str(self._value)
