from typing import Optional

from qtpy.QtCore import Qt
from qtpy.QtCore import Signal as pyqtSignal
from qtpy.QtWidgets import (
    QComboBox,
    QDialog,
    QHBoxLayout,
    QLineEdit,
    QPushButton,
    QWidget,
)


class LambdaBreakpointDialog(QDialog):

    accepted = pyqtSignal(str, str, name="accepted")

    def __init__(
        self,
        lambdas: list[str],
        parent: Optional['QWidget'] = None,
        flags: Qt.WindowFlags | Qt.WindowType = Qt.WindowFlags(),
    ) -> None:
        super().__init__(parent, flags)

        self.lambdaSelectWidget = QComboBox()
        self.lambdaSelectWidget.addItems(lambdas)

        self.valuesWidget = QLineEdit()

        okButton = QPushButton('OK')
        okButton.clicked.connect(self.signalAccepted)
        cancelButton = QPushButton('Cancel')
        cancelButton.clicked.connect(self.close)

        hboxLayout = QHBoxLayout()
        hboxLayout.addWidget(self.lambdaSelectWidget, 0)
        hboxLayout.addWidget(self.valuesWidget, 1)
        hboxLayout.addWidget(okButton)
        hboxLayout.addWidget(cancelButton)
        self.setLayout(hboxLayout)

        self.show()

    def signalAccepted(self) -> None:
        lambda_name = self.lambdaSelectWidget.currentText()
        enteredValues = self.valuesWidget.text()
        self.accepted.emit(lambda_name, enteredValues)
        self.close()
