from typing import Optional

from qtpy.QtCore import Qt
from qtpy.QtCore import Signal as pyqtSignal
from qtpy.QtWidgets import (
    QComboBox,
    QDialog,
    QHBoxLayout,
    QLineEdit,
    QPushButton,
    QWidget,
)

from simudator.core import Module


class ModuleStateDialog(QDialog):
    accepted = pyqtSignal(str, str, str, name='okSignal')

    def __init__(
        self,
        module: Module,
        parent: Optional['QWidget'] = None,
        flags: Qt.WindowFlags | Qt.WindowType = Qt.WindowFlags(),
    ) -> None:
        super().__init__(parent, flags)

        self.module = module
        states = module.get_gui_state()

        # Remove the 'name' state of the module from the drop down menu so
        # that the user cannot e.g. edit or add a breakpoint of a module
        states.pop('name')

        # Set up the drop down menu for selecting a state of the module
        # to perform an action to
        self.stateSelectWidget = QComboBox()
        self.stateSelectWidget.addItems(states.keys())
        self.stateSelectWidget.activated.connect(self.updateState)

        # Set up a text field for entering the value used for the e.g. editing
        # the selected state
        self.valueWidget = QLineEdit()

        # Set up buttons
        self.okButton = QPushButton('OK')
        self.okButton.clicked.connect(self.signalOK)
        self.cancelButton = QPushButton('Cancel')
        self.cancelButton.clicked.connect(self.close)

        # Set up the layout of the widget
        self.HBoxLayout = QHBoxLayout()
        self.HBoxLayout.addWidget(self.stateSelectWidget, 0)
        self.HBoxLayout.addWidget(self.valueWidget, 1)
        self.HBoxLayout.addWidget(self.okButton, 2)
        self.HBoxLayout.addWidget(self.cancelButton, 3)
        self.setLayout(self.HBoxLayout)

        # Show the widget
        self.updateState()
        self.show()

    def updateState(self) -> None:
        selectedState = self.stateSelectWidget.currentText()
        value = self.module.get_gui_state()[selectedState]
        self.valueWidget.setText(str(value))

    def signalOK(self) -> None:
        module_name = self.module.get_state()['name']
        selectedState = self.stateSelectWidget.currentText()
        enteredValue = self.valueWidget.text()
        self.accepted.emit(module_name, selectedState, enteredValue)
        self.close()

    def close(self) -> bool:
        return super().close()
