import time

from qtpy.QtCore import QRunnable

class RunThread(QRunnable):
    """
    This class is used to run the simulated cpu several ticks or continuously on
    a seperate thread. This allows the user to interact with the GUI while the 
    simulation is running.

    After each CPU tick, this thread will emit to its given QT signal so that
    the GUI can update itself and possibly inform the user of when the execution 
    has halted.
    """

    def __init__(self, cpu, signal, delay: float, run_continuously=True, 
                 step_asm=False, steps=0):
        super().__init__()
        self.cpu = cpu
        self.signal = signal
        self.run_continuously = run_continuously
        self.steps = steps
        self.delay = delay
        self.step_asm = step_asm

    def run(self):
        if self.step_asm:

            while self.steps > 0:
                self.cpu.do_tick()

                # We only care about asm instructions
                if self.cpu.is_new_instruction():
                    self.steps -= 1

                self.signal.emit(1)
                time.sleep(self.delay)

                if self.cpu.is_stopped:
                    break

        else:
            if self.run_continuously:
                self.cpu.unstop()
                while not self.cpu.is_stopped:
                    self.cpu.do_tick()
                    self.signal.emit(1)
                    time.sleep(self.delay)

            else:
                for _ in range(self.steps):
                    self.cpu.do_tick()
                    self.signal.emit(1)
                    time.sleep(self.delay)

                    if self.cpu.is_stopped:
                        break


        # Signal end of execution as having run 0 ticks
        self.signal.emit(0)
