import math

from qtpy.QtCore import Qt
from qtpy.QtGui import QBrush
from qtpy.QtWidgets import QAction, QGraphicsRectItem

from simudator.core.module import Module
from simudator.core.signal import Signal
from simudator.gui.module_graphics_item.module_graphics_item import ModuleGraphicsItem
from simudator.gui.orientation import Orientation
from simudator.gui.port_graphics_item import PortGraphicsItem


class BusGraphicsItem(ModuleGraphicsItem):
    """
    Graphics item for bus module.
    """

    WIDTH = 10
    PAIR_MARGIN = 60
    PORT_MARGIN = 20

    def __init__(self, module: Module, signal_pairs: [(Signal)]):
        self.signal_pairs = signal_pairs
        super().__init__(module)

    def draw_graphics_item(self) -> None:
        # Calculate height according to number of signals and states
        height = (math.ceil(len(self.signal_pairs) / 2)) * (
            self.PAIR_MARGIN + self.PORT_MARGIN
        ) - self.PAIR_MARGIN

        # Create base background rectangle
        self.baserect = QGraphicsRectItem(0, 0, self.WIDTH, height, self)
        self.baserect.setBrush(QBrush(Qt.black))

        # add port pairs
        for index, signals in enumerate(self.signal_pairs):
            # x coordinates and orientation will be
            # diffrent for every other signal pair
            x_pos = self.WIDTH
            orientation = Orientation.RIGHT
            if index % 2 == 0:
                x_pos = 0
                orientation = Orientation.LEFT

            # If either signal is none dont create a port for it
            if signals[0] is not None:
                output_port = PortGraphicsItem(signals[0], orientation, self)

                output_port.setPos(
                    x_pos, (index // 2) * (self.PAIR_MARGIN + self.PORT_MARGIN)
                )
                self.ports.append(output_port)

            if signals[1] is not None:
                input_port = PortGraphicsItem(signals[1], orientation, self)

                input_port.setPos(
                    x_pos,
                    (index // 2) * (self.PAIR_MARGIN + self.PORT_MARGIN)
                    + self.PORT_MARGIN,
                )
                self.ports.append(input_port)

    def update(self):
        pass

    def shouldIgnoreAction(self, action: QAction) -> bool:
        """
        Disables left clicking, which would normally allow the user to
        enter a breakpoint or edit the state.
        """
        return True

    def getName(self) -> str:
        return self.module.get_name()
