from qtpy.QtWidgets import QAction, QGraphicsRectItem, QGraphicsSimpleTextItem

from simudator.gui.module_graphics_item.module_graphics_item import ModuleGraphicsItem
from simudator.gui.orientation import Orientation
from simudator.gui.port_graphics_item import PortGraphicsItem


class AluGraphicsItem(ModuleGraphicsItem):
    """
    Graphics module for mia's ALU module.
    """

    RECT_WIDTH = 120
    RECT_HEIGHT = 60

    def draw_graphics_item(self):
        # Base rectangle
        self.baserect = QGraphicsRectItem(0, 0, self.RECT_WIDTH, self.RECT_HEIGHT, self)

        # Create name lable
        name_text = QGraphicsSimpleTextItem(self.state["name"], self)
        name_text.setPos(
            self.RECT_WIDTH / 2 - len(self.state["name"]) * self._CHAR_LEN,
            self.RECT_HEIGHT / 2 - len(self.state["name"]) * self._CHAR_LEN,
        )

        # make port for input a
        port_a = PortGraphicsItem(self.module.signals["in_input_a"], Orientation.UP, self)
        port_a.setPos(self.RECT_WIDTH * 3 / 4, 0)
        self.ports.append(port_a)

        # make port for input b
        port_b = PortGraphicsItem(self.module.signals["in_input_b"], Orientation.UP, self)
        port_b.setPos(self.RECT_WIDTH / 4, 0)
        self.ports.append(port_b)

        # make port for output
        output_port = PortGraphicsItem(
            self.module.signals["out_result"], Orientation.DOWN, self
        )
        output_port.setPos(self.RECT_WIDTH / 2, self.RECT_HEIGHT)
        self.ports.append(output_port)

    def shouldIgnoreAction(self, action: QAction) -> bool:
        """
        Disables left clicking, which would normally allow the user to
        enter a breakpoint or edit the state.
        """
        return True
