from qtpy.QtWidgets import QGraphicsRectItem, QGraphicsSimpleTextItem

from simudator.core.module import Module
from simudator.gui.module_graphics_item.module_graphics_item import ModuleGraphicsItem


class IrGraphicsItem(ModuleGraphicsItem):
    """
    Graphics module for mia's IR module.
    """

    # This module has two parts, a small rectangle with the basic IR info
    # and a larger with a more detailed representation
    SMALL_RECT_HEIGHT = 30
    SMALL_RECT_WIDTH = 70

    BIG_RECT_HEIGHT = 60
    BIG_RECT_WIDTH = 180

    TEXT_WIDTH_MARGIN = 5
    TEXT_HEIGHT_MARGIN = 8
    BIG_TEXT_HEIGHT_MARGIN = 12

    def __init__(self, module: Module, name: str = None):
        super().__init__(module, name)
        self.draw_ports()

    def draw_graphics_item(self):
        # Add small rect
        QGraphicsRectItem(0, 0, self.SMALL_RECT_WIDTH, self.SMALL_RECT_HEIGHT, self)

        # Add text for small rect
        self.small_text = QGraphicsSimpleTextItem("", self)
        self.small_text.setPos(self.TEXT_WIDTH_MARGIN, self.TEXT_HEIGHT_MARGIN)

        # Add big rect
        self.baserect = QGraphicsRectItem(
            0, self.SMALL_RECT_HEIGHT, self.BIG_RECT_WIDTH, self.BIG_RECT_HEIGHT, self
        )

        # Add text verbose text for big rect
        self.big_text = QGraphicsSimpleTextItem("", self)
        self.big_text.setPos(
            self.TEXT_WIDTH_MARGIN, self.SMALL_RECT_HEIGHT + self.BIG_TEXT_HEIGHT_MARGIN
        )

        self.update()

    def update(self):
        # Update small text
        instr = self.module.instruction
        self.small_text.setText(f"IR: {instr:04x}")

        # Update larg text
        # TODO: do we need to AND the values? The ir module is already
        # doing that when it recives its updates.
        op_field = instr >> 12
        grx_field = (instr >> 10) & 0b11
        m_field = (instr >> 8) & 0b11
        adr_field = instr & (2**8 - 1)

        instr_text = (
            "op:     "
            + "grx:  "
            + "m:   "
            + "adr:\n"
            + f"{op_field:04b}  "
            + f"{grx_field:02b}    "
            + f"{m_field:02b}   "
            + f"{adr_field:08b}"
        )
        self.big_text.setText(instr_text)
