from simudator.core.modules.register import IntegerRegister
from simudator.core.signal import Signal
from simudator.processor.mia.modules.mia_bus_connect import MiaBusConnector


class AR(IntegerRegister, MiaBusConnector):
    """
    Register for saving AlU calculations in MIA.
    """

    # Python does not allow multiple inherintence if more than one of the
    # parent classes uses __slots__. Thus we also includes the __slots__
    # from MiaBusConnector.
    __slots__ = ("bus_id", "bus_control_s")

    def __init__(
        self,
        alu_input_signal: Signal,
        alu_output_signal: Signal,
        bus_output_signal: Signal,
        bus_control_signal: Signal,
        bus_id: int,
        bit_length: int,
        name="AR",
    ) -> None:

        IntegerRegister.__init__(
            self, alu_input_signal, bus_output_signal, bit_length, name=name
        )
        MiaBusConnector.__init__(self, bus_control_signal, bus_id)

        self.signals["out_alu"] = alu_output_signal

    def output_register(self) -> None:
        # always update alu
        self.signals["out_alu"].update_value(self._value)

        # only update bus when asked too
        if self.write_to_bus():
            IntegerRegister.output_register(self)
        else:
            self.signals["out_content"].update_value(None)

    def update_logic(self) -> None:
        if self.write_to_bus():
            IntegerRegister.output_register(self)
        else:
            self.signals["out_content"].update_value(None)

    def print_module(self):
        print(
            "",
            self.name,
            "\n -----",
            "\n value: ",
            hex(self._value),
            "\n bit length: ",
            self._bit_length,
        )
