from simudator.core.modules.demux import Demux
from simudator.core.processor import Processor
from simudator.core.signal import Signal


def test_default_name():
    cpu = Processor()
    from_demux = Signal(cpu)
    input_s = Signal(cpu)
    outputs = []
    bit_length = 5
    demux = Demux(from_demux, bit_length, input_s, outputs)

    assert demux.name == "demux"


def test_name():
    cpu = Processor()
    from_demux = Signal(cpu)
    input_s = Signal(cpu)
    outputs = []
    bit_length = 5
    demux = Demux(from_demux, bit_length, input_s, outputs, "Demux")

    assert demux.name == "Demux"


def test_get_state():
    cpu = Processor()
    from_demux = Signal(cpu)
    input_s = Signal(cpu)
    outputs = []
    bit_length = 5
    demux = Demux(from_demux, bit_length, input_s, outputs, "Demux", 7)

    state = demux.get_state()

    assert state["name"] == "Demux"
    assert state["value"] == 7
    assert state["bit_length"] == 5


def test_set_state():
    state = dict()
    state["name"] = "Demux"
    state["value"] = 6
    state["bit_length"] = 2

    cpu = Processor()
    from_demux = Signal(cpu)
    input_s = Signal(cpu)
    outputs = []
    bit_length = 5
    demux = Demux(from_demux, bit_length, input_s, outputs, "Demux", 7)

    demux.set_state(state)

    state = demux.get_state()
    assert state["name"] == "Demux"
    assert state["value"] == 6
    assert state["bit_length"] == 2


def test_writing_to_output():
    # Init required modules
    cpu = Processor()
    from_demux_s = Signal(cpu)
    input_s = Signal(cpu)
    output_1_s = Signal(cpu)
    output_2_s = Signal(cpu)
    output_3_s = Signal(cpu)
    outputs = [
        output_1_s,
        output_2_s,
        output_3_s,
    ]
    bit_length = 5
    demux = Demux(from_demux_s, bit_length, input_s, outputs)

    # add demux to cpu
    cpu.add_module(demux)

    # update the control signal and the input signal, the 'correct'
    # output signal should receive the input signal, the others
    # should not be affected
    from_demux_s.update_value(0)
    outputs[0].update_value(0)
    outputs[1].update_value(0)
    outputs[2].update_value(0)
    input_s.update_value(1)
    cpu.do_tick()

    assert outputs[0].get_value() == 1
    assert outputs[1].get_value() == 0
    assert outputs[2].get_value() == 0


def test_throw_away_bits():
    # Init required modules
    cpu = Processor()
    from_demux_s = Signal(cpu)
    input_s = Signal(cpu)
    output_1_s = Signal(cpu)
    outputs = [
        output_1_s,
    ]
    bit_length = 2
    demux = Demux(from_demux_s, bit_length, input_s, outputs)

    # add mux to the cpu
    cpu.add_module(demux)

    # update the control signal and the input signal value,
    # the output signal should receive the value 3 since the
    # mux only has two bits and it is receiving the value 1111
    from_demux_s.update_value(0)
    input_s.update_value(15)
    cpu.do_tick()

    assert outputs[0].get_value() == 3
