from simudator.core.modules.memory import Memory
from simudator.core.processor import Processor
from simudator.core.signal import Signal


def test_memory():
    cpu = Processor()
    input_s = Signal(cpu, value=0)
    output_s = Signal(cpu, value=0)
    control_s = Signal(cpu, value=0)
    adress_s = Signal(cpu, value=0)
    memory = Memory(input_s, output_s, control_s, adress_s, 2)
    cpu.add_module(memory)

    # Test that the memory is not written to before a new clock pulse
    control_s.update_value(1)
    input_s.update_value(1)
    memory.update_logic()  # Simulate update at end of current clock pulse
    assert output_s.get_value() == 0

    # Test that the memory has been written to after the new clock pulse
    cpu.do_tick()
    assert output_s.get_value() == 1

    # Test that reading from the adress does not write to it
    control_s.update_value(0)
    input_s.update_value(0)
    memory.update_logic()  # Simulate update at the end of current clock pulse
    cpu.do_tick()
    assert output_s.get_value() == 1

    # Test that reading from another adress works
    adress_s.update_value(1)
    memory.update_logic()
    assert output_s.get_value() == 0
    cpu.do_tick()
    assert output_s.get_value() == 0
