from simudator.core.processor import Processor
from simudator.core.signal import Signal
from simudator.processor.mia.modules.ar import AR
from simudator.processor.mia.modules.asr import ASR
from simudator.processor.mia.modules.hr import HR


def test_asr():
    cpu = Processor()

    bus_input_signal = Signal(cpu)
    memory_output_signal = Signal(cpu)
    bus_control_signal = Signal(cpu)

    asr = ASR(bus_input_signal, memory_output_signal, bus_control_signal, 1, 8, "ASR")
    cpu.add_module(asr)

    # test send value from bus
    bus_input_signal.update_value(21)
    bus_control_signal.update_value((0, 1))
    cpu.do_tick()
    cpu.do_tick()

    assert memory_output_signal.get_value() == 21

    # test dont send value to bus
    bus_input_signal.update_value(17)
    bus_control_signal.update_value((0, 3))
    cpu.do_tick()
    cpu.do_tick()

    assert memory_output_signal.get_value() == 21


def test_ar():
    cpu = Processor()

    alu_input_signal = Signal(cpu)
    alu_output_signal = Signal(cpu)
    bus_output_signal = Signal(cpu)
    bus_control_signal = Signal(cpu)

    ar = AR(
        alu_input_signal,
        alu_output_signal,
        bus_output_signal,
        bus_control_signal,
        1,
        16,
        "AR",
    )

    cpu.add_module(ar)

    # test send value from alu to bus
    alu_input_signal.update_value(10)
    bus_control_signal.update_value((1, 0))
    cpu.do_tick()
    cpu.do_tick()

    assert bus_output_signal.get_value() == 10
    assert alu_output_signal.get_value() == 10

    # test send none to bus
    alu_input_signal.update_value(22)
    bus_control_signal.update_value((0, 2))
    cpu.do_tick()
    cpu.do_tick()

    assert bus_output_signal.get_value() is None


def test_hr():
    cpu = Processor()

    alu_input_signal = Signal(cpu)
    alu_output_signal = Signal(cpu)
    bus_input_signal = Signal(cpu)
    bus_output_signal = Signal(cpu)
    bus_control_signal = Signal(cpu)

    hr = HR(
        alu_input_signal,
        alu_output_signal,
        bus_input_signal,
        bus_output_signal,
        bus_control_signal,
        1,
        16,
        "HR",
    )

    cpu.add_module(hr)

    # test send value from alu to bus
    alu_input_signal.update_value(10)
    bus_control_signal.update_value((1, 0))
    cpu.do_tick()
    cpu.do_tick()

    assert bus_output_signal.get_value() == 10
    assert alu_output_signal.get_value() == 10

    # test send none to bus
    alu_input_signal.update_value(22)
    bus_control_signal.update_value((0, 2))
    cpu.do_tick()
    cpu.do_tick()

    assert bus_output_signal.get_value() is None

    # test send to alu from bus
    bus_input_signal.update_value(52)
    bus_control_signal.update_value((0, 1))
    cpu.do_tick()
    cpu.do_tick()

    assert alu_output_signal.get_value() == 52
    assert bus_output_signal.get_value() is None

    # try get from bus again
    alu_input_signal.update_value(None)
    bus_control_signal.update_value((1, 0))
    cpu.do_tick()
    cpu.do_tick()

    assert bus_output_signal.get_value() == 52
