from simudator.core.processor import Processor
from simudator.core.signal import Signal
from simudator.processor.mia.modules.micro_memory import MicroMemory


def test_alu_field():
    cpu = Processor()
    alu_s = Signal(cpu, value=0)
    bus_control_s = Signal(cpu, value=0)
    bus_constant_s = Signal(cpu, value=0)
    s_s = Signal(cpu, value=0)
    p_s = Signal(cpu, value=0)
    lc_control_s = Signal(cpu, value=0)
    lc_uadr = Signal(cpu, value=0)
    upc_s = Signal(cpu, value=0)
    upc_control_s = Signal(cpu, value=0)
    upc_uadr_s = Signal(cpu, value=0)
    z_flag_s = Signal(cpu, value=0)
    n_flag_s = Signal(cpu, value=0)
    c_flag_s = Signal(cpu, value=0)
    o_flag_s = Signal(cpu, value=0)
    l_flag_s = Signal(cpu, value=0)
    uM = MicroMemory(
        alu_s,
        bus_control_s,
        bus_constant_s,
        s_s,
        p_s,
        lc_control_s,
        lc_uadr,
        upc_s,
        upc_control_s,
        upc_uadr_s,
        z_flag_s,
        n_flag_s,
        c_flag_s,
        o_flag_s,
        l_flag_s,
    )
    cpu.add_module(uM)

    # Test that the value in the ALU field of the current instruction is
    # outputted onto the ALU control signal
    alu_test_instruction = 0b1111000000000000000000000
    state = uM.get_state()
    state["memory"] = [alu_test_instruction]
    uM.set_state(state)
    uM.update_logic()
    assert alu_s.get_value() == 0b1111


def test_bus_fields():
    cpu = Processor()
    alu_s = Signal(cpu, value=0)
    bus_control_s = Signal(cpu, value=0)
    bus_constant_s = Signal(cpu, value=0)
    s_s = Signal(cpu, value=0)
    p_s = Signal(cpu, value=0)
    lc_control_s = Signal(cpu, value=0)
    lc_uadr = Signal(cpu, value=0)
    upc_s = Signal(cpu, value=0)
    upc_control_s = Signal(cpu, value=0)
    upc_uadr_s = Signal(cpu, value=0)
    z_flag_s = Signal(cpu, value=0)
    n_flag_s = Signal(cpu, value=0)
    c_flag_s = Signal(cpu, value=0)
    o_flag_s = Signal(cpu, value=0)
    l_flag_s = Signal(cpu, value=0)
    uM = MicroMemory(
        alu_s,
        bus_control_s,
        bus_constant_s,
        s_s,
        p_s,
        lc_control_s,
        lc_uadr,
        upc_s,
        upc_control_s,
        upc_uadr_s,
        z_flag_s,
        n_flag_s,
        c_flag_s,
        o_flag_s,
        l_flag_s,
    )
    cpu.add_module(uM)

    # Test that the right control signal is sent for which modules should
    # read from and write to the bus according to the TB & FB instruction
    # fields
    tb_fb_instruction = 0b0000100001000000000000000  # TB: 0b100, FB: 0b001
    state = uM.get_state()
    state["memory"] = [tb_fb_instruction]
    uM.set_state(state)
    uM.update_logic()
    assert bus_control_s.get_value() == (0b100, 0b001)

    # Test the special instruction where the TB instruction field is 0b111
    # ALU: 0b0100 TB: 0b111, bit 8: 0, bit 9-24: only 1s
    tb_fb_instruction = 0b0100111011111111111111111
    state = uM.get_state()
    state["memory"] = [tb_fb_instruction]
    uM.set_state(state)
    uM.update_logic()
    assert alu_s.get_value() == 0b0100  # Should be the ALU field
    assert bus_constant_s.get_value() == 2**16 - 1  # Should be bit 9-24
    assert bus_control_s.get_value() == (0, 0)  # Bus shouldn't do anything
    assert p_s.get_value() == 0  # PC shouldn't do anything
    assert lc_control_s.get_value() == 0  # LC shouldn't do anything


def test_s_field():
    cpu = Processor()
    alu_s = Signal(cpu, value=0)
    bus_control_s = Signal(cpu, value=0)
    bus_constant_s = Signal(cpu, value=0)
    s_s = Signal(cpu, value=0)
    p_s = Signal(cpu, value=0)
    lc_control_s = Signal(cpu, value=0)
    lc_uadr = Signal(cpu, value=0)
    upc_s = Signal(cpu, value=0)
    upc_control_s = Signal(cpu, value=0)
    upc_uadr_s = Signal(cpu, value=0)
    z_flag_s = Signal(cpu, value=0)
    n_flag_s = Signal(cpu, value=0)
    c_flag_s = Signal(cpu, value=0)
    o_flag_s = Signal(cpu, value=0)
    l_flag_s = Signal(cpu, value=0)
    uM = MicroMemory(
        alu_s,
        bus_control_s,
        bus_constant_s,
        s_s,
        p_s,
        lc_control_s,
        lc_uadr,
        upc_s,
        upc_control_s,
        upc_uadr_s,
        z_flag_s,
        n_flag_s,
        c_flag_s,
        o_flag_s,
        l_flag_s,
    )
    cpu.add_module(uM)

    test_instruction = 0b0000000000100000000000000  # S: 1, all other bits: 0
    state = uM.get_state()
    state["memory"] = [test_instruction]
    uM.set_state(state)

    uM.update_logic()
    assert s_s.get_value() == 1


def test_p_field():
    cpu = Processor()
    alu_s = Signal(cpu, value=0)
    bus_control_s = Signal(cpu, value=0)
    bus_constant_s = Signal(cpu, value=0)
    s_s = Signal(cpu, value=0)
    p_s = Signal(cpu, value=0)
    lc_control_s = Signal(cpu, value=0)
    lc_uadr = Signal(cpu, value=0)
    upc_s = Signal(cpu, value=0)
    upc_control_s = Signal(cpu, value=0)
    upc_uadr_s = Signal(cpu, value=0)
    z_flag_s = Signal(cpu, value=0)
    n_flag_s = Signal(cpu, value=0)
    c_flag_s = Signal(cpu, value=0)
    o_flag_s = Signal(cpu, value=0)
    l_flag_s = Signal(cpu, value=0)
    uM = MicroMemory(
        alu_s,
        bus_control_s,
        bus_constant_s,
        s_s,
        p_s,
        lc_control_s,
        lc_uadr,
        upc_s,
        upc_control_s,
        upc_uadr_s,
        z_flag_s,
        n_flag_s,
        c_flag_s,
        o_flag_s,
        l_flag_s,
    )
    cpu.add_module(uM)

    test_instruction = 0b0000000000010000000000000  # P: 1, all other bits: 0
    state = uM.get_state()
    state["memory"] = [test_instruction]
    uM.set_state(state)

    uM.update_logic()
    assert p_s.get_value() == 1


def test_lc_field():
    cpu = Processor()
    alu_s = Signal(cpu, value=0)
    bus_control_s = Signal(cpu, value=0)
    bus_constant_s = Signal(cpu, value=0)
    s_s = Signal(cpu, value=0)
    p_s = Signal(cpu, value=0)
    lc_control_s = Signal(cpu, value=0)
    lc_uadr = Signal(cpu, value=0)
    upc_s = Signal(cpu, value=0)
    upc_control_s = Signal(cpu, value=0)
    upc_uadr_s = Signal(cpu, value=0)
    z_flag_s = Signal(cpu, value=0)
    n_flag_s = Signal(cpu, value=0)
    c_flag_s = Signal(cpu, value=0)
    o_flag_s = Signal(cpu, value=0)
    l_flag_s = Signal(cpu, value=0)
    uM = MicroMemory(
        alu_s,
        bus_control_s,
        bus_constant_s,
        s_s,
        p_s,
        lc_control_s,
        lc_uadr,
        upc_s,
        upc_control_s,
        upc_uadr_s,
        z_flag_s,
        n_flag_s,
        c_flag_s,
        o_flag_s,
        l_flag_s,
    )
    cpu.add_module(uM)

    # Test that the LC field is outputted onto the LC control signal
    test_instruction = 0b1111111111110011111111111  # LC: 00, all other bits are 1s
    state = uM.get_state()
    state["memory"] = [test_instruction]
    uM.set_state(state)
    uM.update_logic()
    assert lc_control_s.get_value() == 0b00

    # Test the special case where the LC field is 0b11
    test_instruction = 0b0000000000001100001111111  # LC: 11, uADR: 1111111
    state = uM.get_state()
    state["memory"] = [test_instruction]
    uM.set_state(state)
    uM.update_logic()
    assert lc_control_s.get_value() == 0b11  # Should be equal to LC
    assert lc_uadr.get_value() == 0b1111111  # Should be equal to uADR


def test_seq_field():
    cpu = Processor()
    alu_s = Signal(cpu, value=0)
    bus_control_s = Signal(cpu, value=0)
    bus_constant_s = Signal(cpu, value=0)
    s_s = Signal(cpu, value=0)
    p_s = Signal(cpu, value=0)
    lc_control_s = Signal(cpu, value=0)
    lc_uadr = Signal(cpu, value=0)
    upc_s = Signal(cpu, value=0)
    upc_control_s = Signal(cpu, value=0)
    upc_uadr_s = Signal(cpu, value=0)
    z_flag_s = Signal(cpu, value=0)
    n_flag_s = Signal(cpu, value=0)
    c_flag_s = Signal(cpu, value=0)
    o_flag_s = Signal(cpu, value=0)
    l_flag_s = Signal(cpu, value=0)
    uM = MicroMemory(
        alu_s,
        bus_control_s,
        bus_constant_s,
        s_s,
        p_s,
        lc_control_s,
        lc_uadr,
        upc_s,
        upc_control_s,
        upc_uadr_s,
        z_flag_s,
        n_flag_s,
        c_flag_s,
        o_flag_s,
        l_flag_s,
    )
    cpu.add_module(uM)

    # Test basic uPC control --------------------------------------------------
    test_instructions = [
        0b0000000000001100001111111,  # SEQ: 0000 => uPC += 1
        0b0000000000001100010000000,  # SEQ: 0001 => uPC = K1
        0b0000000000001100101111111,  # SEQ: 0010 => uPC = K2
        0b0000000000001100110000000,
    ]  # SEQ: 0011 => uPC = 0
    state = uM.get_state()
    state["memory"] = test_instructions
    uM.set_state(state)

    # For each test instruction, make sure uM outputs the appropriate control
    # signal to uPC. For the basic uPC control instructions the value of the
    # SEQ field is the correct control signal value
    for index, instr in enumerate(test_instructions):
        upc_s.update_value(index)
        uM.update_logic()
        assert upc_control_s.get_value() == (instr >> 7) & 0b1111

    # Test unconditional jump -------------------------------------------------
    test_instruction = 0b0000000000000001011111111  # SEQ: 0101, uADR: 1111111
    state["memory"] = [test_instruction]
    uM.set_state(state)
    upc_s.update_value(0)
    uM.update_logic()
    # The uPC control signal for loading uADR into uPC is 0b100
    assert upc_control_s.get_value() == 0b100  # Control signal => uPC=uADR
    assert upc_uadr_s.get_value() == 0b1111111  # Should be uADR

    # Test conditional jumps --------------------------------------------------

    # A list of each possible conditional jump to be tested
    # A jump is written as a tuple (SEQ, flag_signal, cond_val) where:
    #   SEQ = the bit pattern of the SEQ instruction field of the jump,
    #   flag_signal = the signal of the flag tested for the jump
    #   cond_val = the value that the flag should have for the jump to execute
    test_tups = [
        (0b0100, z_flag_s, 0),
        (0b1000, z_flag_s, 1),
        (0b1001, n_flag_s, 1),
        (0b1010, c_flag_s, 1),
        (0b1011, o_flag_s, 1),
        (0b1100, l_flag_s, 1),
        (0b1101, c_flag_s, 0),
        (0b1110, o_flag_s, 0),
    ]

    # Create the corresponding instructions for every conditional jump
    # The uADR field is set to the index of the instruction + 1
    test_instructions = []
    for index, test_tup in enumerate(test_tups):
        instruction = (test_tup[0] << 7) + index + 1
        test_instructions.append(instruction)

    state["memory"] = test_instructions
    uM.set_state(state)
    upc_s.update_value(0)

    for index, test_tup in enumerate(test_tups):
        flag_signal = test_tup[1]
        cond_val = test_tup[2]
        uADR = index + 1
        upc_s.update_value(index)

        # Test conditional jump with the condition being true
        # The uPC control signal should be 100 which corresponds to uPC = uADR
        flag_signal.update_value(cond_val)
        uM.update_logic()
        assert upc_control_s.get_value() == 0b100  # 100 => uPC = uADR
        assert upc_uadr_s.get_value() == uADR

        # Test the conditional jump withthe condition being false
        # The uPC control signal should be 000 which corresponds to uPC += 1
        flag_signal.update_value(not cond_val)
        uM.update_logic()
        assert upc_control_s.get_value() == 0b000  # 000 => uPC += 1

    # Test subroutine jumps ---------------------------------------------------
    # SEQ: 0110, uADR: 1111111
    subroutine_jump_instr = 0b0000000000000001101111111
    # SEQ: 0111
    subroutine_return_instr = 0b0000000000000001110000000
    state["memory"] = [subroutine_jump_instr, subroutine_return_instr]
    uM.set_state(state)

    upc_s.update_value(0)
    uM.update_logic()
    assert upc_control_s.get_value() == 0b110
    assert upc_uadr_s.get_value() == 0b1111111

    upc_s.update_value(1)
    uM.update_logic()
    assert upc_control_s.get_value() == 0b101

    # Test halt ---------------------------------------------------------------
    test_instruction = 0b0000000000000011110000000
    state["memory"] = [test_instruction]
    uM.set_state(state)

    upc_s.update_value(0)
    uM.update_logic()
    # uPC should be reset to 0, corresponding to control signal 011
    assert upc_control_s.get_value() == 0b011
    # TODO: Test for some sort of HALT signal
